@@ -17,4 +17,4 @@ pub use self::selectors::{BadSelector, Selector, SelectorPart, Selectors};
 pub use self::string::CssString;
 pub use self::value::{Value, ValueMap, ValueToMapError};
 
-pub(crate) use self::util::{is_function_name, is_not};
+pub(crate) use self::util::{is_calc_name, is_function_name, is_not};
