@@ -1,5 +1,5 @@
 //! This module provides `handle_body` (and internally `handle_item`),
-//! that does most of the work for [`super::Format::write_root`].
+//! that does most of the work for [`crate::input::Context::transform`].
 
 // https://github.com/rust-lang/rust-clippy/issues/7846
 // https://users.rust-lang.org/t/using-an-option-mut-t-in-a-loop-clippy-complains/72481/2
@@ -7,15 +7,10 @@
 
 use super::cssbuf::{CssBuf, CssHead};
 use crate::css::{BodyItem, Comment, Import, Property, Rule, Selectors};
-use crate::error::Error;
-use crate::file_context::FileContext;
-use crate::parser::Parsed;
-use crate::sass::{
-    get_global_module, CallArgs, Expose, Function, Item, MixinDecl, Name,
-    UseAs,
-};
+use crate::input::{Context, Loader, Parsed, SourceKind};
+use crate::sass::{get_global_module, Expose, Item, UseAs};
 use crate::value::ValueRange;
-use crate::ScopeRef;
+use crate::{Error, Invalid, ScopeRef};
 use std::io::Write;
 
 pub fn handle_parsed(
@@ -24,7 +19,7 @@ pub fn handle_parsed(
     rule: Option<&mut Rule>,
     buf: &mut CssBuf,
     scope: ScopeRef,
-    file_context: &impl FileContext,
+    file_context: &mut Context<impl Loader>,
 ) -> Result<(), Error> {
     match items {
         Parsed::Scss(items) => {
@@ -45,7 +40,7 @@ fn handle_body(
     rule: Option<&mut Rule>,
     buf: &mut CssBuf,
     scope: ScopeRef,
-    file_context: &impl FileContext,
+    file_context: &mut Context<impl Loader>,
 ) -> Result<(), Error> {
     let mut rule = rule;
     for b in items {
@@ -67,27 +62,25 @@ fn handle_item(
     rule: Option<&mut Rule>,
     buf: &mut CssBuf,
     scope: ScopeRef,
-    file_context: &impl FileContext,
+    file_context: &mut Context<impl Loader>,
 ) -> Result<(), Error> {
     let format = scope.get_format();
     match item {
         Item::Use(ref name, ref as_n, ref with, ref pos) => {
-            let name = name.evaluate(scope.clone())?;
-            let module = if let Some(module) = get_global_module(name.value())
-            {
+            let name = name.evaluate(scope.clone())?.take_value();
+            let module = if let Some(module) = get_global_module(&name) {
                 if !with.is_empty() {
                     return Err(Error::BadCall(
-                        "Error: Built-in modules can\'t be configured."
-                            .into(),
+                        "Built-in modules can\'t be configured.".into(),
                         pos.clone(),
                         None,
                     ));
                 }
                 module
             } else if let Some(sourcefile) =
-                file_context.find_file_use(name.value(), pos.clone())?
+                file_context.find_file(&name, SourceKind::Use(pos.clone()))?
             {
-                head.load_module(
+                let module = head.load_module(
                     sourcefile.path(),
                     |head| {
                         let module = ScopeRef::new_global(format);
@@ -101,7 +94,7 @@ fn handle_item(
                                 value.do_evaluate(scope.clone(), true)
                             })?;
                             if module.get_or_none(name).is_none() {
-                                module.define(name.clone(), &value);
+                                module.define(name.clone(), value)?;
                             } else {
                                 return Err(Error::error(
                                     "The same variable may only be configured once.",
@@ -117,33 +110,33 @@ fn handle_item(
                             file_context,
                         )?;
                         Ok(module)
-                    })?
+                    })?;
+                file_context.unlock_loading(&sourcefile);
+                module
             } else {
                 return Err(Error::BadCall(
-                    "Error: Can't find stylesheet to import.".into(),
+                    "Can't find stylesheet to import.".into(),
                     pos.clone(),
                     None,
                 ));
             };
-            scope.do_use(module, name.value(), as_n, &Expose::All)?;
+            scope.do_use(module, &name, as_n, &Expose::All)?;
         }
         Item::Forward(ref name, ref as_n, ref expose, ref with, ref pos) => {
-            let name = name.evaluate(scope.clone())?;
-            let module = if let Some(module) = get_global_module(name.value())
-            {
+            let name = name.evaluate(scope.clone())?.take_value();
+            let module = if let Some(module) = get_global_module(&name) {
                 if !with.is_empty() {
                     return Err(Error::BadCall(
-                        "Error: Built-in modules can\'t be configured."
-                            .into(),
+                        "Built-in modules can\'t be configured.".into(),
                         pos.clone(),
                         None,
                     ));
                 }
                 module
-            } else if let Some(sourcefile) =
-                file_context.find_file_use(name.value(), pos.clone())?
+            } else if let Some(sourcefile) = file_context
+                .find_file(&name, SourceKind::Forward(pos.clone()))?
             {
-                head.load_module(
+                let module = head.load_module(
                     sourcefile.path(),
                     |head| {
                         let module = ScopeRef::new_global(format);
@@ -157,7 +150,7 @@ fn handle_item(
                                 value.do_evaluate(scope.clone(), true)
                             })?;
                             if module.get_or_none(name).is_none() {
-                                module.define(name.clone(), &value);
+                                module.define(name.clone(), value)?;
                             } else {
                                 return Err(Error::error(
                                     "The same variable may only be configured once.",
@@ -173,21 +166,23 @@ fn handle_item(
                             file_context,
                         )?;
                         Ok(module)
-                    })?
+                    });
+                file_context.unlock_loading(&sourcefile);
+                module?
             } else {
                 return Err(Error::S(format!("Module {} not found", name)));
             };
-            scope.forward().do_use(module, name.value(), as_n, expose)?;
+            scope.forward().do_use(module, &name, as_n, expose)?;
         }
         Item::Import(ref names, ref args, ref pos) => {
             let mut rule = rule;
             'name: for name in names {
+                let name = name.evaluate(scope.clone())?;
                 if args.is_null() {
-                    let x = name.evaluate(scope.clone())?;
+                    let x = name.value();
                     if let Some(sourcefile) = file_context
-                        .find_file_import(x.value(), pos.clone())?
+                        .find_file(x, SourceKind::Import(pos.clone()))?
                     {
-                        scope.lock_loading(sourcefile.path(), pos.clone())?;
                         match sourcefile.parse()? {
                             Parsed::Scss(items) => {
                                 let mut thead = CssHead::new();
@@ -214,23 +209,22 @@ fn handle_item(
                                 }
                             }
                         }
-                        scope.unlock_loading(sourcefile.path());
+                        file_context.unlock_loading(&sourcefile);
                         continue 'name;
                     }
-                    if !(x.value().starts_with("http://")
-                        || x.value().starts_with("https://")
-                        || x.value().starts_with("//")
-                        || x.value().ends_with(".css")
-                        || x.is_css_url())
+                    if !(x.starts_with("http://")
+                        || x.starts_with("https://")
+                        || x.starts_with("//")
+                        || x.ends_with(".css")
+                        || name.is_css_url())
                     {
                         return Err(Error::BadCall(
-                            "Error: Can't find stylesheet to import.".into(),
+                            "Can't find stylesheet to import.".into(),
                             pos.clone(),
                             None,
                         ));
                     }
                 }
-                let name = name.evaluate(scope.clone())?;
                 let args = args.evaluate(scope.clone())?;
                 let import = Import::new(name, args);
                 if let Some(ref mut rule) =
@@ -262,9 +256,10 @@ fn handle_item(
             buf.join(sub);
         }
         Item::AtRule {
-            ref name,
-            ref args,
-            ref body,
+            name,
+            args,
+            body,
+            pos: _,
         } => {
             buf.do_separate();
             buf.do_indent_no_nl();
@@ -307,52 +302,40 @@ fn handle_item(
             ref val,
             default,
             global,
+            ref pos,
         } => {
             let val = val.do_evaluate(scope.clone(), true)?;
-            scope.set_variable(name.into(), val, *default, *global);
+            scope
+                .set_variable(name.clone(), val, *default, *global)
+                .map_err(|e| e.at(pos.clone()))?;
         }
-        Item::FunctionDeclaration(ref name, ref args, ref pos, ref body) => {
+        Item::FunctionDeclaration(ref name, ref body) => {
             if name == "calc"
                 || name == "element"
                 || name == "expression"
                 || name == "url"
             {
                 // Ok, this is cheating for the test suite ...
-                let p = pos.clone().opt_back("@function ");
-                return Err(Error::InvalidFunctionName(p));
+                let p = body.decl.clone().opt_back("@function ");
+                return Err(Invalid::FunctionName.at(p));
             }
-            scope.define_function(
-                name.into(),
-                Function::closure(
-                    args.clone(),
-                    pos.clone(),
-                    scope.clone(),
-                    body.clone(),
-                ),
-            );
+            check_body(&body.body, BodyContext::Function)?;
+            scope.define_function(name.into(), body.closure(&scope).into());
         }
-        Item::Return(_) => {
-            return Err(Error::S(
-                "Return not allowed in plain context".into(),
-            ));
+        Item::Return(_, ref pos) => {
+            return Err(Invalid::AtRule.at(pos.clone()));
         }
 
-        Item::MixinDeclaration(ref name, ref args, ref body, ref pos) => {
-            scope.define_mixin(
-                name.into(),
-                MixinDecl::new(args, &scope, body, pos),
-            )
+        Item::MixinDeclaration(ref name, ref body) => {
+            check_body(&body.body, BodyContext::Mixin)?;
+            scope.define_mixin(name.into(), body.closure(&scope).into())
         }
         Item::MixinCall(ref name, ref args, ref body, ref pos) => {
             if let Some(mixin) = scope.get_mixin(&name.into()) {
-                let mixin = mixin.get(
-                    name,
-                    scope.clone(),
-                    args,
-                    pos,
-                    file_context,
-                )?;
-                mixin.define_content(&scope, body, pos);
+                let mixin = mixin
+                    .get(scope.clone(), args, pos, file_context)
+                    .map_err(|e| e.called_from(pos, name))?;
+                mixin.define_content(&scope, body);
                 handle_parsed(
                     mixin.body,
                     head,
@@ -362,61 +345,51 @@ fn handle_item(
                     file_context,
                 )
                 .map_err(|e: Error| match e {
-                    Error::BadArguments(msg, decl) => {
-                        let pos = pos.in_call(name);
-                        Error::BadCall(msg, pos, Some(decl))
-                    }
-                    Error::AtError(msg, _pos) => {
-                        let msg = format!("Error: {}", msg);
-                        Error::BadCall(msg, pos.clone(), None)
+                    Error::Invalid(err, _) => err.at(pos.clone()),
+                    Error::BadCall(msg, pos, p2) => {
+                        Error::BadCall(msg, pos.in_call(name), p2)
                     }
                     e => {
                         let pos = pos.in_call(name);
-                        Error::BadCall(e.to_string(), pos, None)
+                        Error::BadCall(format!("{:?}", e), pos, None)
                     }
                 })?;
             } else {
                 return Err(Error::BadCall(
-                    "Error: Undefined mixin.".into(),
+                    "Undefined mixin.".into(),
                     pos.clone(),
                     None,
                 ));
             }
         }
-        Item::Content(pos) => {
-            if let Some(content) =
-                scope.get_mixin(&Name::from_static("%%BODY%%"))
-            {
-                if !content.is_no_body() {
-                    let mixin = content.get(
-                        "@content",
-                        scope,
-                        &CallArgs::new(vec![])?,
-                        pos,
-                        file_context,
-                    )?;
-                    handle_parsed(
-                        mixin.body,
-                        head,
-                        rule,
-                        buf,
-                        mixin.scope,
-                        file_context,
-                    )?;
-                }
+        Item::Content(args, pos) => {
+            if let Some(content) = scope.get_content() {
+                let mixin = content
+                    .get(scope, args, pos, file_context)
+                    .map_err(|e| e.called_from(pos, "@content"))?;
+                handle_parsed(
+                    mixin.body,
+                    head,
+                    rule,
+                    buf,
+                    mixin.scope,
+                    file_context,
+                )?;
             }
         }
 
         Item::IfStatement(ref cond, ref do_if, ref do_else) => {
             let cond = cond.evaluate(scope.clone())?.is_true();
             let items = if cond { do_if } else { do_else };
+            check_body(items, BodyContext::Control)?;
             handle_body(items, head, rule, buf, scope, file_context)?;
         }
         Item::Each(ref names, ref values, ref body) => {
+            check_body(body, BodyContext::Control)?;
             let mut rule = rule;
             let pushed = scope.store_local_values(names);
             for value in values.evaluate(scope.clone())?.iter_items() {
-                scope.define_multi(names, &value);
+                scope.define_multi(names, value)?;
                 handle_body(
                     body,
                     head,
@@ -440,10 +413,11 @@ fn handle_item(
                 to.evaluate(scope.clone())?,
                 *inclusive,
             )?;
+            check_body(body, BodyContext::Control)?;
             let mut rule = rule;
             for value in range {
                 let scope = ScopeRef::sub(scope.clone());
-                scope.define(name.clone(), &value);
+                scope.define(name.clone(), value)?;
                 handle_body(
                     body,
                     head,
@@ -455,6 +429,7 @@ fn handle_item(
             }
         }
         Item::While(ref cond, ref body) => {
+            check_body(body, BodyContext::Control)?;
             let mut rule = rule;
             let scope = ScopeRef::sub(scope);
             while cond.evaluate(scope.clone())?.is_true() {
@@ -476,13 +451,14 @@ fn handle_item(
             eprintln!("WARNING: {}", value.evaluate(scope)?.format(format));
         }
         Item::Error(ref value, ref pos) => {
-            return Err(Error::AtError(
+            return Err(Invalid::AtError(
                 value.evaluate(scope)?.format(format).to_string(),
-                pos.clone(),
-            ));
+            )
+            .at(pos.clone()));
         }
 
         Item::Rule(ref selectors, ref body) => {
+            check_body(body, BodyContext::Rule)?;
             if rule.is_none() {
                 buf.do_separate();
             }
@@ -509,7 +485,9 @@ fn handle_item(
                     rule.push(Property::new(name.value().into(), v).into());
                 }
             } else {
-                return Err(Error::S("Global property not allowed".into()));
+                return Err(Error::error(
+                    "Declarations may only be used within style rules.",
+                ));
             }
         }
         Item::CustomProperty(ref name, ref value) => {
@@ -523,13 +501,14 @@ fn handle_item(
                     ));
                 }
             } else {
-                return Err(Error::S(
-                    "Global custom property not allowed".into(),
+                return Err(Error::error(
+                    "Global custom property not allowed",
                 ));
             }
         }
         Item::NamespaceRule(ref name, ref value, ref body) => {
             if let Some(rule) = rule {
+                check_body(body, BodyContext::NsRule)?;
                 let value = value.evaluate(scope.clone())?;
                 let name = name.evaluate(scope.clone())?;
                 if !value.is_null() {
@@ -556,13 +535,13 @@ fn handle_item(
                     })
                 }
                 if !sub.is_empty() {
-                    return Err(Error::S(
-                        "Unexpected content in namespace rule".into(),
+                    return Err(Error::error(
+                        "Unexpected content in namespace rule",
                     ));
                 }
             } else {
-                return Err(Error::S(
-                    "Global namespaced property not allowed".into(),
+                return Err(Error::error(
+                    "Global namespaced property not allowed",
                 ));
             }
         }
@@ -581,3 +560,93 @@ fn handle_item(
     }
     Ok(())
 }
+
+#[derive(Clone, Copy, PartialEq, Eq)]
+enum BodyContext {
+    Mixin,
+    Function,
+    Control,
+    Rule,
+    NsRule,
+}
+
+fn check_body(body: &[Item], context: BodyContext) -> Result<(), Error> {
+    for item in body {
+        match item {
+            Item::Forward(_, _, _, _, pos) => {
+                return Err(Invalid::AtRule.at(pos.clone()));
+            }
+            Item::Use(_, _, _, pos) => {
+                return Err(Invalid::AtRule.at(pos.clone()));
+            }
+            Item::MixinDeclaration(.., ref decl) => {
+                let pos = decl.decl.clone().opt_back("@mixin ");
+                match context {
+                    BodyContext::Mixin => {
+                        return Err(Invalid::MixinInMixin.at(pos));
+                    }
+                    BodyContext::Control => {
+                        return Err(Invalid::MixinInControl.at(pos));
+                    }
+                    BodyContext::Rule => (), // This is ok
+                    _ => {
+                        return Err(Invalid::AtRule.at(pos.opt_trail_ws()));
+                    }
+                }
+            }
+            Item::FunctionDeclaration(_, ref body) => {
+                let pos = body.decl.clone().opt_back("@function ");
+                match context {
+                    BodyContext::Mixin => {
+                        return Err(Invalid::FunctionInMixin.at(pos));
+                    }
+                    BodyContext::Control => {
+                        return Err(Invalid::FunctionInControl.at(pos));
+                    }
+                    BodyContext::Rule => (), // This is ok
+                    _ => {
+                        return Err(Invalid::AtRule.at(pos.opt_trail_ws()));
+                    }
+                }
+            }
+            Item::Return(_, ref pos) if context != BodyContext::Function => {
+                return Err(Invalid::AtRule.at(pos.clone()));
+            }
+            Item::AtRule {
+                name,
+                args: _,
+                body: _,
+                pos,
+            } if context != BodyContext::Rule => {
+                if !name
+                    .single_raw()
+                    .map(|name| CSS_AT_RULES.contains(&name))
+                    .unwrap_or(false)
+                {
+                    return Err(Invalid::AtRule.at(pos.clone()));
+                }
+            }
+            _ => (),
+        }
+    }
+    Ok(())
+}
+
+const CSS_AT_RULES: [&str; 16] = [
+    "charset",
+    "color-profile",
+    "counter-style",
+    "document",
+    "font-face",
+    "font-feature-values",
+    "import",
+    "keyframes",
+    "layer",
+    "media",
+    "namespace",
+    "page",
+    "property",
+    "scroll-timeline",
+    "supports",
+    "viewport",
+];
