@@ -20,7 +20,7 @@ pub(crate) mod util;
 pub mod value;
 
 pub use error::ParseError;
-pub use pos::{SourceName, SourcePos};
+pub use pos::SourcePos;
 
 use self::formalargs::{call_args, formal_args};
 use self::selectors::selectors;
@@ -34,7 +34,8 @@ use self::util::{
 use self::value::{
     dictionary, function_call, single_value, value_expression,
 };
-use crate::sass::{FormalArgs, Item, Name, Selectors, Value};
+use crate::input::SourceName;
+use crate::sass::{Callable, FormalArgs, Item, Name, Selectors, Value};
 use crate::value::ListSeparator;
 #[cfg(test)]
 use crate::value::{Numeric, Rgba, Unit};
@@ -52,94 +53,12 @@ use nom::multi::{
 use nom::sequence::{delimited, pair, preceded, terminated};
 use nom::IResult;
 use nom_locate::LocatedSpan;
-use std::convert::TryFrom;
-use std::io::Read;
 use std::str::{from_utf8, Utf8Error};
 
 pub type Span<'a> = LocatedSpan<&'a [u8], &'a SourceName>;
 /// A Parsing Result; ok gives a span for the rest of the data and a parsed T.
 type PResult<'a, T> = IResult<Span<'a>, T>;
 
-/// A supported input format.
-///
-/// Rsass handles the scss format and raw css.
-/// TODO: In the future, the sass format may also be supported.
-#[derive(Clone, Copy, Debug, PartialEq, Eq)]
-enum SourceFormat {
-    /// The scss format is the main input format.
-    Scss,
-    /// The css format
-    Css,
-}
-
-impl TryFrom<&str> for SourceFormat {
-    type Error = Error;
-    fn try_from(name: &str) -> Result<SourceFormat, Error> {
-        if name.ends_with(".scss") {
-            Ok(SourceFormat::Scss)
-        } else if name.ends_with(".css") {
-            Ok(SourceFormat::Css)
-        } else {
-            Err(Error::error(format!("Unknown source format: {:?}", name)))
-        }
-    }
-}
-
-/// The full data of a source file.
-///
-/// This type contains both the contents (as a `Vec<u8>`) of a file
-/// and information on from where (and why) it was loaded.
-/// You can create a SourceFile with the [`SourceFile::read`]
-/// constructor, but normally you will get one from a
-/// [`FileContext`][crate::FileContext].
-pub struct SourceFile {
-    data: Vec<u8>,
-    source: SourceName,
-    format: SourceFormat,
-}
-
-impl SourceFile {
-    /// Create a SourceFile from something readable and a name.
-    pub fn read<T: Read>(
-        file: &mut T,
-        source: SourceName,
-    ) -> Result<Self, Error> {
-        let format = SourceFormat::try_from(source.name())?;
-        let mut data = vec![];
-        file.read_to_end(&mut data)
-            .map_err(|e| Error::Input(source.name().to_string(), e))?;
-        Ok(SourceFile {
-            data,
-            source,
-            format,
-        })
-    }
-
-    /// Parse this scss source.
-    pub fn parse(&self) -> Result<Parsed, Error> {
-        let data = Span::new_extra(&self.data, &self.source);
-        match self.format {
-            SourceFormat::Scss => {
-                Ok(Parsed::Scss(ParseError::check(sassfile(data))?))
-            }
-            SourceFormat::Css => {
-                Ok(Parsed::Css(ParseError::check(css::file(data))?))
-            }
-        }
-    }
-    pub(crate) fn path(&self) -> &str {
-        self.source.name()
-    }
-}
-/// Parsed source that is either css or sass data.
-#[derive(Clone, Debug)]
-pub enum Parsed {
-    /// Raw css data.
-    Css(Vec<crate::css::Item>),
-    /// Sass (scss) data.
-    Scss(Vec<crate::sass::Item>),
-}
-
 pub fn code_span(value: &[u8]) -> Span {
     use lazy_static::lazy_static;
     lazy_static! {
@@ -177,14 +96,7 @@ fn test_parse_value_data_2() -> Result<(), Error> {
     Ok(())
 }
 
-/// Parse scss data from a buffer.
-///
-/// Returns a vec of the top level items of the file (or an error message).
-pub fn parse_scss_data(data: &[u8]) -> Result<Vec<Item>, ParseError> {
-    ParseError::check(sassfile(code_span(data)))
-}
-
-fn sassfile(input: Span) -> PResult<Vec<Item>> {
+pub(crate) fn sassfile(input: Span) -> PResult<Vec<Item>> {
     preceded(
         opt(tag("\u{feff}".as_bytes())),
         map(
@@ -203,7 +115,7 @@ fn top_level_item(input: Span) -> PResult<Item> {
         b"$" => variable_declaration2(input),
         b"/*" => comment_item(input),
         b"@" => at_rule2(input),
-        b"" => rule(input),
+        b"" => alt((variable_declaration_mod, rule))(input),
         _ => unreachable!(),
     }
 }
@@ -273,11 +185,25 @@ fn mixin_call2(input: Span) -> PResult<Item> {
     let (rest, n2) = opt(preceded(tag("."), name))(rest)?;
     let name = n2.map(|n2| format!("{}.{}", n1, n2)).unwrap_or(n1);
     let (rest, _) = opt_spacelike(rest)?;
-    let (rest, args) = terminated(opt(call_args), opt_spacelike)(rest)?;
-    let (end, body) = terminated(
-        opt(body_block),
-        terminated(opt_spacelike, opt(tag(";"))),
-    )(rest)?;
+    let (rest0, args) = terminated(opt(call_args), opt_spacelike)(rest)?;
+    let (rest, t) = alt((tag("using"), tag("{"), tag("")))(rest0)?;
+    let (end, body) = match *t.fragment() {
+        b"using" => {
+            let (end, args) = preceded(opt_spacelike, formal_args)(rest)?;
+            let (rest, body) = preceded(opt_spacelike, body_block)(end)?;
+            let decl = SourcePos::from_to(rest0, end);
+            (rest, Some(Callable::new(args, body, decl)))
+        }
+        b"{" => {
+            let (rest, body) = body_block(rest0)?;
+            let decl = SourcePos::from_to(rest0, rest);
+            (rest, Some(Callable::no_args(body, decl)))
+        }
+        _ => {
+            let (rest, _) = opt(tag(";"))(rest)?;
+            (rest, None)
+        }
+    };
     let pos = SourcePos::from_to(input, rest).opt_back("@include ");
     Ok((
         end,
@@ -297,7 +223,7 @@ fn at_rule2(input0: Span) -> PResult<Item> {
         "error" => {
             let (end, v) = value_expression(input)?;
             let (rest, _) = opt(tag(";"))(end)?;
-            let pos = SourcePos::from_to(input, end).opt_back("@error ");
+            let pos = SourcePos::from_to(input0, end).opt_back("@");
             Ok((rest, Item::Error(v, pos)))
         }
         "for" => for_loop2(input),
@@ -307,12 +233,13 @@ fn at_rule2(input0: Span) -> PResult<Item> {
         "import" => import2(input),
         "include" => mixin_call2(input),
         "mixin" => mixin_declaration2(input),
-        "return" => return_stmt2(input),
+        "return" => return_stmt2(input0, input),
         "use" => use2(input0),
         "warn" => map(expression_argument, Item::Warn)(input),
         "while" => while_loop2(input),
         _ => {
             let (input, name) = sass_string(input0)?;
+            let pos = SourcePos::from_to(input0, input).opt_back("@");
             let (input, args) = opt(media_args)(input)?;
             let (input, body) = preceded(
                 opt(ignore_space),
@@ -324,6 +251,7 @@ fn at_rule2(input0: Span) -> PResult<Item> {
                     name,
                     args: args.unwrap_or(Value::Null),
                     body,
+                    pos,
                 },
             ))
         }
@@ -501,31 +429,39 @@ fn mixin_declaration2(input: Span) -> PResult<Item> {
     let (rest, args) = opt(formal_args)(rest)?;
     let (end, body) = preceded(opt_spacelike, body_block)(rest)?;
     let args = args.unwrap_or_else(FormalArgs::none);
-    let pos = SourcePos::from_to(input, rest);
-    Ok((end, Item::MixinDeclaration(name, args, body, pos)))
+    let decl = SourcePos::from_to(input, rest);
+    Ok((
+        end,
+        Item::MixinDeclaration(name, Callable { args, body, decl }),
+    ))
 }
 
 fn function_declaration2(input: Span) -> PResult<Item> {
     let (end, name) = terminated(name, opt_spacelike)(input)?;
     let (end, args) = formal_args(end)?;
     let (rest, body) = preceded(opt_spacelike, body_block)(end)?;
-    let pos = SourcePos::from_to(input, end);
-    Ok((rest, Item::FunctionDeclaration(name, args, pos, body)))
+    let decl = SourcePos::from_to(input, end);
+    Ok((
+        rest,
+        Item::FunctionDeclaration(name, Callable { args, body, decl }),
+    ))
 }
 
-fn return_stmt2(input: Span) -> PResult<Item> {
+fn return_stmt2<'a>(input0: Span<'_>, input: Span<'a>) -> PResult<'a, Item> {
     let (input, v) =
         delimited(opt_spacelike, value_expression, opt_spacelike)(input)?;
+    let pos = SourcePos::from_to(input0, input).opt_back("@");
     let (input, _) = opt(tag(";"))(input)?;
-    Ok((input, Item::Return(v)))
+    Ok((input, Item::Return(v, pos)))
 }
 
 /// The "rest" of an `@content` statement is just an optional terminator
 fn content_stmt2(input: Span) -> PResult<Item> {
     let (rest, _) = opt_spacelike(input)?;
+    let (rest, args) = opt(call_args)(rest)?;
     let (rest, _) = opt(tag(";"))(rest)?;
     let pos = SourcePos::from_to(input, rest);
-    Ok((rest, Item::Content(pos)))
+    Ok((rest, Item::Content(args.unwrap_or_default(), pos)))
 }
 
 fn custom_property(input: Span) -> PResult<Item> {
@@ -599,16 +535,37 @@ fn body_block2(input: Span) -> PResult<Vec<Item>> {
     Ok((input, v))
 }
 
-#[cfg(test)] // TODO: Or remove this?
 fn variable_declaration(input: Span) -> PResult<Item> {
     preceded(tag("$"), variable_declaration2)(input)
 }
 
-fn variable_declaration2(input: Span) -> PResult<Item> {
+fn variable_declaration_mod(input: Span) -> PResult<Item> {
+    map(
+        pair(terminated(name, tag(".")), variable_declaration),
+        |(module, decl)| match decl {
+            Item::VariableDeclaration {
+                name,
+                val,
+                default,
+                global,
+                pos,
+            } => Item::VariableDeclaration {
+                name: format!("{}.{}", module, name).into(),
+                val,
+                default,
+                global,
+                pos: pos.opt_back(&format!("{}.", module)),
+            },
+            _ => unreachable!(),
+        },
+    )(input)
+}
+
+fn variable_declaration2(input0: Span) -> PResult<Item> {
     let (input, name) = terminated(
-        name,
+        map(name, Name::from),
         delimited(opt_spacelike, tag(":"), opt_spacelike),
-    )(input)?;
+    )(input0)?;
     let (input, val) = terminated(value_expression, opt_spacelike)(input)?;
     let (input, (default, global)) = fold_many0(
         terminated(
@@ -621,14 +578,15 @@ fn variable_declaration2(input: Span) -> PResult<Item> {
         || (false, false),
         |(default, global), (d, g)| (default || d, global || g),
     )(input)?;
-    let (input, _) = semi_or_end(input)?;
+    let (trail, _) = semi_or_end(input)?;
     Ok((
-        input,
+        trail,
         Item::VariableDeclaration {
             name,
             val,
             default,
             global,
+            pos: SourcePos::from_to(input0, input).opt_back("$"),
         },
     ))
 }
@@ -654,12 +612,12 @@ fn string(v: &str) -> Value {
 #[test]
 fn if_with_no_else() {
     assert_eq!(
-        check_parse!(if_statement, b"@if true { p { color: black; } }\n"),
+        check_parse!(if_statement, b"@if true { p { border: solid; } }\n"),
         Item::IfStatement(
             Value::True,
             vec![Item::Rule(
                 selectors(code_span(b"p")).unwrap().1,
-                vec![Item::Property("color".into(), Value::black())],
+                vec![Item::Property("border".into(), string("solid"))],
             )],
             vec![],
         ),
@@ -697,47 +655,66 @@ fn test_property_2() {
 
 #[test]
 fn test_variable_declaration_simple() {
-    assert_eq!(
-        check_parse!(variable_declaration, b"$foo: bar;"),
+    match check_parse!(variable_declaration, b"$foo: bar;") {
         Item::VariableDeclaration {
-            name: "foo".into(),
-            val: string("bar"),
-            default: false,
-            global: false,
-        },
-    )
+            name,
+            val,
+            default,
+            global,
+            pos: _,
+        } => {
+            assert_eq!(
+                (name, val, default, global),
+                ("foo".into(), string("bar"), false, false)
+            )
+        }
+        _ => panic!(),
+    }
 }
 
 #[test]
 fn test_variable_declaration_global() {
-    assert_eq!(
-        check_parse!(variable_declaration, b"$y: some value !global;"),
+    match check_parse!(variable_declaration, b"$y: some value !global;") {
         Item::VariableDeclaration {
-            name: "y".into(),
-            val: Value::List(
-                vec![string("some"), string("value")],
-                Some(ListSeparator::Space),
-                false,
-            ),
-            default: false,
-            global: true,
-        },
-    )
+            name,
+            val,
+            default,
+            global,
+            pos: _,
+        } => {
+            assert_eq!(
+                (name, val, default, global),
+                (
+                    "y".into(),
+                    Value::List(
+                        vec![string("some"), string("value")],
+                        Some(ListSeparator::Space),
+                        false,
+                    ),
+                    false,
+                    true,
+                )
+            )
+        }
+        _ => panic!(),
+    }
 }
 
 #[test]
 fn test_variable_declaration_default() {
-    assert_eq!(
-        check_parse!(variable_declaration, b"$y: some value !default;"),
+    match check_parse!(variable_declaration, b"$y: value !default;") {
         Item::VariableDeclaration {
-            name: "y".into(),
-            val: Value::List(
-                vec![string("some"), string("value")],
-                Some(ListSeparator::Space),
-                false,
-            ),
-            default: true,
-            global: false,
-        },
-    )
+            name,
+            val,
+            default,
+            global,
+            pos: _,
+        } => {
+            assert_eq!(
+                (name, val, default, global),
+                ("y".into(), string("value"), true, false,)
+            )
+        }
+        _ => panic!(),
+    }
 }
