@@ -11,6 +11,7 @@ macro_rules! name {
 }
 
 mod call_args;
+mod callable;
 mod formal_args;
 mod functions;
 mod item;
@@ -21,8 +22,11 @@ mod string;
 mod value;
 
 pub use self::call_args::CallArgs;
+pub use self::callable::{Call, Callable, Closure};
 pub use self::formal_args::{ArgsError, FormalArgs};
-pub use self::functions::{get_global_module, Function};
+pub use self::functions::{
+    get_global_module, CallError, Function, ResolvedArgs,
+};
 pub use self::item::{Expose, Item, UseAs};
 pub use self::mixin::{Mixin, MixinDecl};
 pub use self::name::Name;
