@@ -1,5 +1,6 @@
 use deunicode::deunicode;
 use hrx_get::Archive;
+use lazy_regex::regex_is_match;
 use std::ffi::OsStr;
 use std::fs::{create_dir, DirEntry, File};
 use std::io::{self, Read, Write};
@@ -24,14 +25,12 @@ fn main() -> Result<(), Error> {
             "core_functions/selector/is_superselector", // not supported
             "core_functions/selector/unify",  // not supported
             "directives/extend", // `@extend` is not supported at all
-            "libsass-todo-issues/issue_1801", // infinite recursion
             "libsass-todo-issues/issue_221262.hrx", // stack overflow
-            "libsass-todo-issues/issue_221260.hrx", // stack overflow
             "libsass-todo-issues/issue_221292.hrx", // stack overflow
             "libsass/unicode-bom/utf-16-big", // rsass only handles utf8
             "libsass/unicode-bom/utf-16-little", // rsass only handles utf8
-            "non_conformant/scss/huge.hrx", // stack overflow in debug mode
-            "non_conformant/scss/multiline_var.hrx", // duplicate rust name
+            "non_conformant/scss/huge", // stack overflow in debug mode
+            "non_conformant/scss/multiline-var.hrx", // duplicate rust name
         ],
     )?;
     Ok(())
@@ -268,6 +267,22 @@ fn spec_hrx_to_test(
             runner = runner.mock_file(name, content);
         }
     }
+    for (name, content) in archive.entries() {
+        if let Some(base) = name.strip_suffix("input.scss") {
+            if references_input(content)
+                || archive.entries().any(|(path, content)| {
+                    path.starts_with(base) && references_input(content)
+                })
+            {
+                writeln!(
+                    rs,
+                    "        .mock_file({:?}, {:#?})",
+                    name, content
+                )?;
+                runner = runner.mock_file(name, content);
+            }
+        }
+    }
     if let Some(p) = precision {
         writeln!(rs, "        .set_precision({})", p)?;
     }
@@ -275,6 +290,10 @@ fn spec_hrx_to_test(
     handle_hrx_part(rs, suite, &archive, "", precision, runner)
 }
 
+fn references_input(content: &str) -> bool {
+    regex_is_match!("['\"]input['\"]", content)
+}
+
 fn handle_hrx_part(
     rs: &mut dyn Write,
     suite: &Path,
