@@ -7,7 +7,6 @@ use num_traits::{
     CheckedAdd, CheckedDiv, CheckedMul, CheckedSub, One, Signed, Zero,
 };
 use std::cmp::Ordering;
-use std::convert::{TryFrom, TryInto};
 use std::fmt::{self, Write};
 use std::ops::{Add, Div, Mul, Neg, Rem, Sub};
 
@@ -421,12 +420,6 @@ impl Number {
         self.value.is_negative()
     }
 
-    /// Returns true if the number is an integer.
-    #[deprecated]
-    pub fn is_integer(&self) -> bool {
-        self.clone().into_integer().is_ok()
-    }
-
     /// Returns true if the number is an integer.
     pub fn into_integer(self) -> Result<i64, Self> {
         fn float_int(s: &f64) -> Option<i64> {
