@@ -2,7 +2,8 @@
 use crate::css::{CssString, Selectors, Value};
 use crate::output::Format;
 use crate::sass::{Expose, Function, Item, MixinDecl, Name, UseAs};
-use crate::{Error, SourcePos};
+use crate::{Error, Invalid, SourcePos};
+use arc_swap::ArcSwapOption;
 use lazy_static::lazy_static;
 use std::collections::BTreeMap;
 use std::ops::Deref;
@@ -70,7 +71,7 @@ impl ScopeRef {
                 Item::Each(ref names, ref values, ref body) => {
                     let s = self.clone();
                     for value in values.evaluate(s.clone())?.iter_items() {
-                        s.define_multi(names, &value);
+                        s.define_multi(names, value)?;
                         if let Some(r) = s.clone().eval_body(body)? {
                             return Ok(Some(r));
                         }
@@ -91,7 +92,7 @@ impl ScopeRef {
                     )?;
                     let s = self.clone();
                     for value in range {
-                        s.define(name.clone(), &value);
+                        s.define(name.clone(), value)?;
                         if let Some(r) = s.clone().eval_body(body)? {
                             return Ok(Some(r));
                         }
@@ -103,12 +104,14 @@ impl ScopeRef {
                     ref val,
                     default,
                     global,
+                    ref pos,
                 } => {
                     let val = val.evaluate(self.clone())?;
-                    self.set_variable(name.into(), val, default, global);
+                    self.set_variable(name.clone(), val, default, global)
+                        .map_err(|e| e.at(pos.clone()))?;
                     None
                 }
-                Item::Return(ref v) => {
+                Item::Return(ref v, _) => {
                     Some(v.do_evaluate(self.clone(), true)?)
                 }
                 Item::While(ref cond, ref body) => {
@@ -130,13 +133,13 @@ impl ScopeRef {
                     None
                 }
                 Item::Error(ref value, ref pos) => {
-                    return Err(Error::AtError(
+                    return Err(Invalid::AtError(
                         value
                             .evaluate(self)?
                             .format(Format::introspect())
                             .to_string(),
-                        pos.clone(),
-                    ));
+                    )
+                    .at(pos.clone()));
                 }
                 Item::None => None,
                 Item::Comment(..) => None,
@@ -182,7 +185,7 @@ impl ScopeRef {
             }
             for (name, value) in &*self.variables.lock().unwrap() {
                 if filter.allow_var(name) {
-                    result.define(name.clone(), value);
+                    result.define(name.clone(), value.clone()).unwrap();
                 }
             }
             result
@@ -217,11 +220,11 @@ pub struct Scope {
     selectors: Option<Selectors>,
     forward: Mutex<Option<ScopeRef>>,
     format: Format,
-    // Set of files currently loading. Only used in the root scope.
-    loading: Mutex<BTreeMap<String, SourcePos>>,
+    /// The thing to use for `@content` in a mixin.
+    content: ArcSwapOption<MixinDecl>,
 }
 
-impl<'a> Scope {
+impl Scope {
     /// Create a new global scope.
     ///
     /// A "global" scope is just a scope that have no parent.
@@ -237,18 +240,14 @@ impl<'a> Scope {
             selectors: None,
             forward: Default::default(),
             format,
-            loading: Default::default(),
+            content: None.into(),
         }
     }
     /// Create a scope for a built-in module.
     pub fn builtin_module(name: &'static str) -> Self {
         let s = Scope::new_global(Default::default());
-        s.set_variable(
-            Name::from_static("@scope_name@"),
-            name.into(),
-            false,
-            false,
-        );
+        s.define(Name::from_static("@scope_name@"), name.into())
+            .unwrap();
         s
     }
     pub(crate) fn get_name(&self) -> String {
@@ -269,7 +268,7 @@ impl<'a> Scope {
             selectors: None,
             forward: Default::default(),
             format,
-            loading: Default::default(),
+            content: None.into(),
         }
     }
     /// Create a new subscope of a given parent with selectors.
@@ -284,7 +283,7 @@ impl<'a> Scope {
             selectors: Some(selectors),
             forward: Default::default(),
             format,
-            loading: Default::default(),
+            content: None.into(),
         }
     }
 
@@ -312,8 +311,8 @@ impl<'a> Scope {
     }
 
     /// Define a none-default, non-global variable.
-    pub fn define(&self, name: Name, val: &Value) {
-        self.set_variable(name, val.clone(), false, false)
+    pub fn define(&self, name: Name, val: Value) -> Result<(), ScopeError> {
+        self.set_variable(name, val, false, false)
     }
 
     /// Define a variable with a value.
@@ -325,17 +324,39 @@ impl<'a> Scope {
         val: Value,
         default: bool,
         global: bool,
-    ) {
+    ) -> Result<(), ScopeError> {
+        if let Some((modulename, name)) = name.split_module() {
+            if let Some(module) = self.get_module(&modulename) {
+                if module
+                    .get_local_or_none(&Name::from_static("@scope_name@"))
+                    .is_some()
+                {
+                    return Err(ScopeError::ModifiedBuiltin);
+                } else {
+                    // Refuse to declare new var from outside module:
+                    let _check_existence = module.get(&name)?;
+                    return module.set_variable(name, val, default, global);
+                }
+            } else {
+                return Err(ScopeError::NoModule(modulename));
+            }
+        }
+        /*if let Some(fwd) = self.opt_forward() {
+            fwd.get_or_none(&name);
+        } else {
+            dbg!("not forwarded");
+        }*/
         if default
             && !matches!(self.get_or_none(&name), Some(Value::Null) | None)
         {
-            return;
+            return Ok(());
         }
         if global {
             self.define_global(name, val);
         } else {
             self.variables.lock().unwrap().insert(name, val);
         }
+        Ok(())
     }
     /// Define a variable in the global scope that is an ultimate
     /// parent of this scope.
@@ -348,11 +369,15 @@ impl<'a> Scope {
     }
     /// Define multiple names from a value that is a list.
     /// Special case: in names is a single name, value is used directly.
-    pub fn define_multi(&self, names: &[Name], value: &Value) {
+    pub(crate) fn define_multi(
+        &self,
+        names: &[Name],
+        value: Value,
+    ) -> Result<(), ScopeError> {
         if names.len() == 1 {
-            self.define(names[0].clone(), value);
+            Ok(self.define(names[0].clone(), value)?)
         } else {
-            let values = value.clone().iter_items();
+            let values = value.iter_items();
             if values.len() > names.len() {
                 panic!(
                     "Expected {} values, but got {}",
@@ -360,13 +385,14 @@ impl<'a> Scope {
                     values.len(),
                 )
             } else {
-                let mut values = values.iter();
+                let mut values = values.into_iter();
                 for name in names {
                     self.define(
                         name.clone(),
-                        values.next().unwrap_or(&Value::Null),
-                    )
+                        values.next().unwrap_or(Value::Null),
+                    )?;
                 }
+                Ok(())
             }
         }
     }
@@ -394,14 +420,13 @@ impl<'a> Scope {
     /// Get the value for a variable (or an error).
     pub fn get(&self, name: &Name) -> Result<Value, ScopeError> {
         if let Some((modulename, name)) = name.split_module() {
-            if let Some(module) = self.get_module(&modulename) {
-                return module.get(&name);
-            } else {
-                return Err(ScopeError::NoModule(modulename));
-            }
+            self.get_module(&modulename)
+                .ok_or(ScopeError::NoModule(modulename))?
+                .get(&name)
+        } else {
+            self.get_local_or_none(name)
+                .ok_or(ScopeError::UndefinedVariable)
         }
-        self.get_local_or_none(name)
-            .ok_or_else(|| ScopeError::Undefined(name.clone()))
     }
 
     /// Copy a set of local variables to a temporary holder
@@ -460,19 +485,16 @@ impl<'a> Scope {
     pub fn get_function(
         &self,
         name: &Name,
-    ) -> Result<Option<Function>, Error> {
+    ) -> Result<Option<Function>, ScopeError> {
         if let Some((modulename, name)) = name.split_module() {
             if let Some(module) = self.get_module(&modulename) {
                 if let Some(f) = module.get_function(&name)? {
                     Ok(Some(f))
                 } else {
-                    Err(Error::error("Undefined function."))
+                    Err(ScopeError::UndefinedFunction)
                 }
             } else {
-                return Err(Error::error(format!(
-                    "There is no module with the namespace {:?}.",
-                    modulename
-                )));
+                Err(ScopeError::NoModule(modulename))
             }
         } else {
             let f = self.functions.lock().unwrap().get(name).cloned();
@@ -536,7 +558,7 @@ impl<'a> Scope {
                 for (name, value) in &*module.variables.lock().unwrap() {
                     let name = format!("{}{}", prefix, name).into();
                     if expose.allow_fun(&name) {
-                        self.define(name, value);
+                        self.define(name, value.clone())?;
                     }
                 }
                 for (name, m) in &*module.mixins.lock().unwrap() {
@@ -555,7 +577,7 @@ impl<'a> Scope {
             self.define_function(name.clone(), function.clone());
         }
         for (name, value) in &*other.variables.lock().unwrap() {
-            self.define(name.clone(), value);
+            self.define(name.clone(), value.clone()).unwrap();
         }
         for (name, m) in &*other.mixins.lock().unwrap() {
             self.define_mixin(name.clone(), m.clone());
@@ -601,36 +623,19 @@ impl<'a> Scope {
             .get_or_insert_with(|| ScopeRef::new_global(self.get_format()))
             .clone()
     }
-    /// Get the forward scope for this scope.
-    ///
-    /// Create a new one if necessary.
+    /// Get the forward scope for this scope, if any.
     pub fn opt_forward(&self) -> Option<ScopeRef> {
         self.forward.lock().unwrap().clone()
     }
-    pub(crate) fn lock_loading(
-        &self,
-        name: &str,
-        pos: SourcePos,
-    ) -> Result<(), Error> {
-        if let Some(parent) = &self.parent {
-            parent.lock_loading(name, pos)
-        } else if let Some(old) = self
-            .loading
-            .lock()
-            .unwrap()
-            .insert(name.into(), pos.clone())
-        {
-            Err(Error::ImportLoop(pos, old))
-        } else {
-            Ok(())
-        }
+
+    pub(crate) fn define_content(&self, body: MixinDecl) {
+        self.content.store(Some(Arc::new(body)));
     }
-    pub(crate) fn unlock_loading(&self, name: &str) {
-        if let Some(parent) = &self.parent {
-            parent.unlock_loading(name)
-        } else {
-            self.loading.lock().unwrap().remove(name);
-        }
+    pub(crate) fn get_content(&self) -> Option<MixinDecl> {
+        self.content
+            .load_full()
+            .map(|c| (*c).clone())
+            .or_else(|| self.parent.as_ref().and_then(|p| p.get_content()))
     }
 }
 
@@ -972,8 +977,8 @@ pub mod test {
             let val = value_expression(code_span(val.as_bytes()));
             scope.define(
                 Name::from_static(name),
-                &ParseError::check(val)?.evaluate(scope.clone())?,
-            );
+                ParseError::check(val)?.evaluate(scope.clone())?,
+            )?;
         }
         let expr =
             terminated(value_expression, tag(";"))(code_span(expression));
@@ -985,12 +990,25 @@ pub mod test {
 }
 
 /// Tried to get something that does not exist from a scope.
+#[derive(Debug)]
 pub enum ScopeError {
     /// Tried to access a module that does not exist.
     NoModule(String),
     /// Tried to access an undefined name.
-    Undefined(Name),
+    UndefinedVariable,
+    /// Undefined function.
+    UndefinedFunction,
+    /// Cannot modify built-in variable.
+    ModifiedBuiltin,
 }
+
+impl ScopeError {
+    /// Make an Error from a ScopeError at a specific position.
+    pub fn at(self, pos: SourcePos) -> Error {
+        Invalid::InScope(self).at(pos)
+    }
+}
+
 use std::fmt::{self, Display};
 impl Display for ScopeError {
     fn fmt(&self, out: &mut fmt::Formatter) -> fmt::Result {
@@ -1000,9 +1018,10 @@ impl Display for ScopeError {
                 "There is no module with the namespace {:?}.",
                 name
             ),
-            ScopeError::Undefined(name) => {
-                // Note: Currently true, but may be used for other things.
-                write!(out, "Undefined variable: \"${}\"", name)
+            ScopeError::UndefinedVariable => "Undefined variable.".fmt(out),
+            ScopeError::UndefinedFunction => "Undefined function.".fmt(out),
+            ScopeError::ModifiedBuiltin => {
+                "Cannot modify built-in variable.".fmt(out)
             }
         }
     }
