@@ -5,4 +5,10 @@ fn runner() -> crate::TestRunner {
     super::runner().with_cwd("media")
 }
 
+mod bubbling;
+
+mod logic;
+
 mod range;
+
+mod test_type;
